/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.chart.tests.engine.util;

import java.util.Calendar;
import junit.framework.TestCase;
import org.eclipse.birt.chart.util.CDateTime;
import org.eclipse.birt.chart.exception.ChartException;

public class CDateTimeTest extends TestCase
{

	CDateTime time, time1;
	CDateTime[] times;

	/**
	 * Construct and initialize any objects that will be used in multiple tests.
	 */
	protected void setUp( ) throws Exception
	{
		super.setUp( );
		time = new CDateTime( 2005, 7, 21, 5, 30, 30 );
		time1 = new CDateTime( 2005, 6, 20 );
		times = new CDateTime[3];
		times[0] = time;
		times[1] = time1;
		times[2] = new CDateTime( 2004, 6, 20 );
	}

	/**
	 * Collect and empty any objects that are defined in the setUp() method.
	 */
	protected void tearDown( ) throws Exception
	{
		super.tearDown( );
		time = null;
		time1 = null;
		times = null;
	}

	/**
	 * Test the existing DataTime object moving back by 'step' units.
	 *
	 */
	public void testBackward( )
	{
		assertEquals( times[2], time1.backward( Calendar.YEAR, 1 ) );
	}
	
	/**
	 * Test the existing DataTime object moving forward by 'step' units.
	 *
	 */
	public void testForward( )
	{
		assertEquals( time1, times[2].forward( Calendar.YEAR, 1 ) );
	}

	/**
	 * Tests the year/month/date/hour/minute/second
	 * associated with this instance.
	 *
	 */
	public void testGetTime( )
	{
		assertEquals( 2005, time.getYear( ) );
		assertEquals( 6, time.getMonth( ) );
		assertEquals( 21, time.getDay( ) );
		assertEquals( 5, time.getHour( ) );
		assertEquals( 30, time.getMinute( ) );
		assertEquals( 30, time.getSecond( ) );
	}

	/**
	 * Test the most significant datetime unit.
	 *
	 */
	public void testGetDifference( )
	{
		assertEquals( 2, CDateTime.getDifference( time, time1 ) );
	}

	/**
	 * Test the default preferred format specifier.
	 *
	 */
	public void testGetPreferredFormat( )
	{
		assertEquals( "yyyy", CDateTime.getPreferredFormat( 1 ) ); //$NON-NLS-1$
		assertEquals( "MMM yyyy", CDateTime.getPreferredFormat( 2 ) ); //$NON-NLS-1$
		assertEquals( "MM-dd-yyyy", CDateTime.getPreferredFormat( 5 ) ); //$NON-NLS-1$
		assertEquals( "MM-dd-yy\nHH:mm", CDateTime.getPreferredFormat( 11 ) ); //$NON-NLS-1$
		assertEquals( "HH:mm:ss", CDateTime.getPreferredFormat( 12 ) ); //$NON-NLS-1$
		assertEquals( "HH:mm:ss", CDateTime.getPreferredFormat( 13 ) ); //$NON-NLS-1$
	}

	/**
	 * Test the difference between two given datetime values.
	 *
	 */
	public void testComputeDifference( )
	{
		assertTrue( CDateTime.computeDifference( time, time, 2 ) == 0 );
		assertTrue( CDateTime.computeDifference( time1, times[2], 5 ) == 365 );
	}

	/**
	 * Test the least significant unit.
	 * 
	 * @throws ChartException
	 */
	public void testComputeUnit( ) throws ChartException
	{
		assertEquals( 13, CDateTime.computeUnit( times ) );
	}

	/**
	 * Tests the number of days.
	 *
	 */
	public void testGetMaximumDaysIn( )
	{
		assertEquals( 366, CDateTime.getMaximumDaysIn( 2000 ) );
		assertEquals( 365, CDateTime.getMaximumDaysIn( 2005 ) );
		assertEquals( 31, CDateTime.getMaximumDaysIn( 0, 2000 ) );
		assertEquals( 29, CDateTime.getMaximumDaysIn( 1, 2000 ) );
		assertEquals( 28, CDateTime.getMaximumDaysIn( 1, 1997 ) );
		assertEquals( 30, CDateTime.getMaximumDaysIn( 5, 2005 ) );
	}

	/**
	 * Test the number of milliseconds in a given unit
	 *
	 */
	public void testInMillis( )
	{
		assertTrue( CDateTime.inMillis( 1 ) == 3.15576E10 );
		assertTrue( CDateTime.inMillis( 2 ) == 2.6298E9 );
		assertTrue( CDateTime.inMillis( 5 ) == 8.64E7 );
		assertTrue( CDateTime.inMillis( 11 ) == 0.0 );
		assertTrue( CDateTime.inMillis( 12 ) == 60000.0 );
		assertTrue( CDateTime.inMillis( 13 ) == 1000.0 );
	}

	/**
	 * Test the value formatted as MM-dd-yyyy HH:mm:ss
	 *
	 */
	public void testParse( )
	{
		assertEquals( time, CDateTime.parse( "07-21-2005 05:30:30" ) ); //$NON-NLS-1$
	}

	public void testToString( )
	{
		assertEquals( "07-21-2005 05:30:30", time.toString( ) ); //$NON-NLS-1$
	}

}
