/*
 * The SEI Software Open Source License, Version 1.0
 *
 * Copyright (c) 2004, Solution Engineering, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Solution Engineering, Inc. (http://www.seisw.com/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 3. The name "Solution Engineering" must not be used to endorse or
 *    promote products derived from this software without prior
 *    written permission. For written permission, please contact
 *    admin@seisw.com.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SOLUTION ENGINEERING, INC. OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 */

package org.eclipse.birt.chart.tests.engine.util;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.eclipse.birt.chart.computation.IPolygon;
import org.eclipse.birt.chart.computation.Point;
import org.eclipse.birt.chart.internal.computations.Clip;
import org.eclipse.birt.chart.internal.computations.Polygon;


/**
 * <code>ClipTest</code> is a suite of unit tests for testing
 * <code>Clip</code>. <code>Clip</code> is a Java conversion of the
 * <i>General Poly Clipper</i> algorithm developed by Alan Murta
 * (gpc@cs.man.ac.uk).
 * 
 * @author Dan Bridenbecker, Solution Engineering, Inc.
 */
public class ClipTest extends TestCase
{

	/**
	 * 
	 * @param name
	 *            Name of test case
	 */
	public ClipTest( String name )
	{
		super( name );
	}

	/**
	 * Provides the ability to run the tests contained here in.
	 * 
	 * @param args
	 *            Command line arguements.
	 */
	public static void main( String args[] )
	{
		junit.textui.TestRunner.run( ClipTest.class );
	}

	/**
	 * Return the suite of tests
	 */
	public static TestSuite suite( )
	{
		return new TestSuite( ClipTest.class );
	}

	/**
	 * Construct and initialize any objects that will be used in multiple tests.
	 * tests. Currently Empty.
	 */
	protected void setUp( )
	{
	}

	protected void tearDown( )
	{
	}

	/**
	 * Test the intersection of two polygons that are completely separate -
	 * result should be empty set.
	 */
	public void testIntersectionEmptySet( )
	{
		Polygon p1 = new Polygon();
		p1.add( new Point( 0.0, 0.0 ) );
		p1.add( new Point( 1.0, 0.0 ) );
		p1.add( new Point( 1.0, 1.0 ) );
		p1.add( new Point( 0.0, 1.0 ) );

		Polygon p2 = new Polygon();
		p2.add( new Point( 2.0, 0.0 ) );
		p2.add( new Point( 3.0, 0.0 ) );
		p2.add( new Point( 3.0, 3.0 ) );
		p2.add( new Point( 2.0, 3.0 ) );

		IPolygon empty = Clip.intersection( p1, p2 );
		assertTrue( empty.isEmpty( ) );
	}

	/**
	 * Test the intersection of two polygons where the second is contained in
	 * the first.
	 */
	public void testIntersectionOneContainsTwo( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 0.0 );
		p1.add( 4.0, 0.0 );
		p1.add( 4.0, 4.0 );
		p1.add( 0.0, 4.0 );

		Polygon p2 = new Polygon();
		p2.add( 2.0, 1.0 );
		p2.add( 3.0, 1.0 );
		p2.add( 3.0, 3.0 );
		p2.add( 2.0, 3.0 );

		IPolygon result = Clip.intersection( p1, p2 );
		assertEquals( p2, result );
	}

	/**
	 * Test the intersection of two polygons where the first is contained in the
	 * second.
	 */
	public void testIntersectionTwoContainsOne( )
	{
		Polygon p1 = new Polygon();
		p1.add( 2.0, 0.0 );
		p1.add( 3.0, 0.0 );
		p1.add( 3.0, 3.0 );
		p1.add( 2.0, 3.0 );

		Polygon p2 = new Polygon();
		p2.add( 0.0, 0.0 );
		p2.add( 4.0, 0.0 );
		p2.add( 4.0, 4.0 );
		p2.add( 0.0, 4.0 );

		IPolygon result = Clip.intersection( p1, p2 );
		assertEquals( p1, result );
	}

	/**
	 * Test the intersection of two polygons that are equal.
	 */
	public void testIntersectionTwoEqual( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 0.0 );
		p1.add( 4.0, 0.0 );
		p1.add( 4.0, 4.0 );
		p1.add( 0.0, 4.0 );

		Polygon p2 = new Polygon();
		p2.add( 0.0, 0.0 );
		p2.add( 4.0, 0.0 );
		p2.add( 4.0, 4.0 );
		p2.add( 0.0, 4.0 );

		IPolygon result = Clip.intersection( p1, p2 );
		assertEquals( p1, result );
	}

	/**
	 * Test the intersection of two rectangles that share one corner and two
	 * partial sides.
	 */
	public void testIntersectionRectCorner1( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 0.0 );
		p1.add( 4.0, 0.0 );
		p1.add( 4.0, 4.0 );
		p1.add( 0.0, 4.0 );

		Polygon p2 = new Polygon();
		p2.add( 0.0, 0.0 );
		p2.add( 1.0, 0.0 );
		p2.add( 1.0, 1.0 );
		p2.add( 0.0, 1.0 );

		IPolygon result = Clip.intersection( p1, p2 );
		assertEquals( p2, result );
	}

	/**
	 * Test the intersection of two rectangles that share one corner and two
	 * partial sides.
	 */
	public void testIntersectionRectCorner2( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 0.0 );
		p1.add( 4.0, 0.0 );
		p1.add( 4.0, 4.0 );
		p1.add( 0.0, 4.0 );

		Polygon p2 = new Polygon();
		p2.add( 3.0, 0.0 );
		p2.add( 4.0, 0.0 );
		p2.add( 4.0, 1.0 );
		p2.add( 3.0, 1.0 );

		IPolygon result = Clip.intersection( p1, p2 );
		assertEquals( p2, result );
	}

	/**
	 * Test the intersection of two rectangles that share one corner and two
	 * partial sides.
	 */
	public void testIntersectionRectCorner3( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 0.0 );
		p1.add( 4.0, 0.0 );
		p1.add( 4.0, 4.0 );
		p1.add( 0.0, 4.0 );

		Polygon p2 = new Polygon();
		p2.add( 3.0, 3.0 );
		p2.add( 4.0, 3.0 );
		p2.add( 4.0, 4.0 );
		p2.add( 3.0, 4.0 );

		IPolygon result = Clip.intersection( p1, p2 );
		assertEquals( p2, result );
	}

	/**
	 * Test the intersection of two rectangles that share one corner and two
	 * partial sides.
	 */
	public void testIntersectionRectCorner4( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 0.0 );
		p1.add( 4.0, 0.0 );
		p1.add( 4.0, 4.0 );
		p1.add( 0.0, 4.0 );

		Polygon p2 = new Polygon();
		p2.add( 0.0, 3.0 );
		p2.add( 1.0, 3.0 );
		p2.add( 1.0, 4.0 );
		p2.add( 0.0, 4.0 );

		IPolygon result = Clip.intersection( p1, p2 );
		assertEquals( p2, result );
	}

	/**
	 * Test the intersection of two rectangles that intersect on corner
	 */
	public void testIntersectionRectInterCorner1( )
	{
		Polygon p1 = new Polygon();
		p1.add( 2.0, 2.0 );
		p1.add( 4.0, 2.0 );
		p1.add( 4.0, 4.0 );
		p1.add( 2.0, 4.0 );

		Polygon p2 = new Polygon();
		p2.add( 1.0, 1.0 );
		p2.add( 3.0, 1.0 );
		p2.add( 3.0, 3.0 );
		p2.add( 1.0, 3.0 );

		Polygon exp = new Polygon();
		exp.add( 2.0, 2.0 );
		exp.add( 3.0, 2.0 );
		exp.add( 3.0, 3.0 );
		exp.add( 2.0, 3.0 );

		IPolygon result = Clip.intersection( p1, p2 );
		assertEquals( exp, result );
	}

	/**
	 * Test the intersection of two rectangles that intersect on corner
	 */
	public void testIntersectionRectInterCorner2( )
	{
		Polygon p1 = new Polygon();
		p1.add( 2.0, 2.0 );
		p1.add( 4.0, 2.0 );
		p1.add( 4.0, 4.0 );
		p1.add( 2.0, 4.0 );

		Polygon p2 = new Polygon();
		p2.add( 3.0, 1.0 );
		p2.add( 5.0, 1.0 );
		p2.add( 5.0, 3.0 );
		p2.add( 3.0, 3.0 );

		Polygon exp = new Polygon();
		exp.add( 3.0, 2.0 );
		exp.add( 4.0, 2.0 );
		exp.add( 4.0, 3.0 );
		exp.add( 3.0, 3.0 );

		IPolygon result = Clip.intersection( p1, p2 );
		assertEquals( exp, result );
	}

	/**
	 * Test the intersection of two rectangles that intersect on corner
	 */
	public void testIntersectionRectInterCorner3( )
	{
		Polygon p1 = new Polygon();
		p1.add( 2.0, 2.0 );
		p1.add( 4.0, 2.0 );
		p1.add( 4.0, 4.0 );
		p1.add( 2.0, 4.0 );

		Polygon p2 = new Polygon();
		p2.add( 3.0, 3.0 );
		p2.add( 5.0, 3.0 );
		p2.add( 5.0, 5.0 );
		p2.add( 3.0, 5.0 );

		Polygon exp = new Polygon();
		exp.add( 3.0, 3.0 );
		exp.add( 4.0, 3.0 );
		exp.add( 4.0, 4.0 );
		exp.add( 3.0, 4.0 );

		IPolygon result = Clip.intersection( p1, p2 );
		assertEquals( exp, result );
	}

	/**
	 * Test the intersection of two rectangles that intersect on corner
	 */
	public void testIntersectionRectInterCorner4( )
	{
		Polygon p1 = new Polygon();
		p1.add( 2.0, 2.0 );
		p1.add( 4.0, 2.0 );
		p1.add( 4.0, 4.0 );
		p1.add( 2.0, 4.0 );

		Polygon p2 = new Polygon();
		p2.add( 1.0, 3.0 );
		p2.add( 3.0, 3.0 );
		p2.add( 3.0, 5.0 );
		p2.add( 1.0, 5.0 );

		Polygon exp = new Polygon();
		exp.add( 2.0, 3.0 );
		exp.add( 3.0, 3.0 );
		exp.add( 3.0, 4.0 );
		exp.add( 2.0, 4.0 );

		IPolygon result = Clip.intersection( p1, p2 );
		assertEquals( exp, result );
	}

	/**
	 * Test the intersection of two rectangles where half of one is contained in
	 * the other and two sides of the inner cross one side of the outer.
	 */
	public void testIntersectionRectInterSide1( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 1.0 );
		p1.add( 4.0, 1.0 );
		p1.add( 4.0, 5.0 );
		p1.add( 0.0, 5.0 );

		Polygon p2 = new Polygon();
		p2.add( 1.0, 0.0 );
		p2.add( 3.0, 0.0 );
		p2.add( 3.0, 2.0 );
		p2.add( 1.0, 2.0 );

		Polygon exp = new Polygon();
		exp.add( 1.0, 1.0 );
		exp.add( 3.0, 1.0 );
		exp.add( 3.0, 2.0 );
		exp.add( 1.0, 2.0 );

		IPolygon result = Clip.intersection( p1, p2 );
		assertEquals( exp, result );
	}

	/**
	 * Test the intersection of two rectangles where half of one is contained in
	 * the other and two sides of the inner cross one side of the outer.
	 */
	public void testIntersectionRectInterSide2( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 1.0 );
		p1.add( 4.0, 1.0 );
		p1.add( 4.0, 5.0 );
		p1.add( 0.0, 5.0 );

		Polygon p2 = new Polygon();
		p2.add( 3.0, 2.0 );
		p2.add( 5.0, 2.0 );
		p2.add( 5.0, 4.0 );
		p2.add( 3.0, 4.0 );

		Polygon exp = new Polygon();
		exp.add( 3.0, 2.0 );
		exp.add( 4.0, 2.0 );
		exp.add( 4.0, 4.0 );
		exp.add( 3.0, 4.0 );

		IPolygon result = Clip.intersection( p1, p2 );
		assertEquals( exp, result );
	}

	/**
	 * Test the intersection of two rectangles where half of one is contained in
	 * the other and two sides of the inner cross one side of the outer.
	 */
	public void testIntersectionRectInterSide3( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 1.0 );
		p1.add( 4.0, 1.0 );
		p1.add( 4.0, 5.0 );
		p1.add( 0.0, 5.0 );

		Polygon p2 = new Polygon();
		p2.add( 1.0, 4.0 );
		p2.add( 3.0, 4.0 );
		p2.add( 3.0, 6.0 );
		p2.add( 1.0, 6.0 );

		Polygon exp = new Polygon();
		exp.add( 1.0, 4.0 );
		exp.add( 3.0, 4.0 );
		exp.add( 3.0, 5.0 );
		exp.add( 1.0, 5.0 );

		IPolygon result = Clip.intersection( p1, p2 );
		assertEquals( exp, result );
	}

	/**
	 * Test the intersection of two rectangles where half of one is contained in
	 * the other and two sides of the inner cross one side of the outer.
	 */
	public void testIntersectionRectInterSide4( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 1.0 );
		p1.add( 4.0, 1.0 );
		p1.add( 4.0, 5.0 );
		p1.add( 0.0, 5.0 );

		Polygon p2 = new Polygon();
		p2.add( -1.0, 2.0 );
		p2.add( 1.0, 2.0 );
		p2.add( 1.0, 4.0 );
		p2.add( -1.0, 4.0 );

		Polygon exp = new Polygon();
		exp.add( 0.0, 2.0 );
		exp.add( 1.0, 2.0 );
		exp.add( 1.0, 4.0 );
		exp.add( 0.0, 4.0 );

		IPolygon result = Clip.intersection( p1, p2 );
		assertEquals( exp, result );
	}

	/**
	 * Test the intersection of two rectangles side by side.
	 */
	public void testIntersectionRectSideBySide( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 1.0 );
		p1.add( 4.0, 1.0 );
		p1.add( 4.0, 5.0 );
		p1.add( 0.0, 5.0 );

		Polygon p2 = new Polygon();
		p2.add( 4.0, 1.0 );
		p2.add( 8.0, 1.0 );
		p2.add( 8.0, 5.0 );
		p2.add( 4.0, 5.0 );

		IPolygon result = Clip.intersection( p1, p2 );
		assertTrue( result.isEmpty() );
	}

	/**
	 * Test the intersection of two complex, non-convex, non-self-intersecting
	 * polygons - 1 on top of two = empty
	 */
	public void testIntersectionPolyOneOnTopOfTwo( )
	{
		Polygon p1 = new Polygon();
		p1.add( 1.0, 4.0 );
		p1.add( 5.0, 4.0 );
		p1.add( 5.0, 9.0 );
		p1.add( 3.0, 7.0 );
		p1.add( 1.0, 9.0 );

		Polygon p2 = new Polygon();
		p2.add( 0.0, 2.0 );
		p2.add( 2.0, 0.0 );
		p2.add( 3.0, 1.0 );
		p2.add( 4.0, 0.0 );
		p2.add( 6.0, 2.0 );
		p2.add( 4.0, 4.0 );
		p2.add( 3.0, 3.0 );
		p2.add( 2.0, 4.0 );

		IPolygon result = Clip.intersection( p1, p2 );
		assertTrue( result.isEmpty( ) );
	}

	/**
	 * Test the intersection of two complex, non-convex, non-self-intersecting
	 * polygons - two sides
	 */
	public void testIntersectionPolyTwoSides( )
	{
		Polygon p1 = new Polygon();
		p1.add( 1.0, 2.0 );
		p1.add( 5.0, 2.0 );
		p1.add( 5.0, 7.0 );
		p1.add( 3.0, 5.0 );
		p1.add( 1.0, 7.0 );

		Polygon p2 = new Polygon();
		p2.add( 0.0, 2.0 );
		p2.add( 2.0, 0.0 );
		p2.add( 3.0, 1.0 );
		p2.add( 4.0, 0.0 );
		p2.add( 6.0, 2.0 );
		p2.add( 4.0, 4.0 );
		p2.add( 3.0, 3.0 );
		p2.add( 2.0, 4.0 );

		Polygon exp = new Polygon();
		exp.add( 1.0, 2.0 );
		exp.add( 5.0, 2.0 );
		exp.add( 5.0, 3.0 );
		exp.add( 4.0, 4.0 );
		exp.add( 3.0, 3.0 );
		exp.add( 2.0, 4.0 );
		exp.add( 1.0, 3.0 );

		IPolygon result = Clip.intersection( p1, p2 );
		// result.print();
		assertEquals( exp, result );
	}

	/**
	 * Test the intersection of two complex, non-convex, non-self-intersecting
	 * polygons - two sides and one vertex - the lower one
	 */
	public void testIntersectionPolyTwoSidesAndLowerVertex( )
	{
		Polygon p1 = new Polygon();
		p1.add( 1.0, 1.0 );
		p1.add( 5.0, 1.0 );
		p1.add( 5.0, 6.0 );
		p1.add( 3.0, 4.0 );
		p1.add( 1.0, 6.0 );

		Polygon p2 = new Polygon();
		p2.add( 0.0, 2.0 );
		p2.add( 2.0, 0.0 );
		p2.add( 3.0, 1.0 );
		p2.add( 4.0, 0.0 );
		p2.add( 6.0, 2.0 );
		p2.add( 4.0, 4.0 );
		p2.add( 3.0, 3.0 );
		p2.add( 2.0, 4.0 );

		Polygon exp = new Polygon();
		exp.add( 1.0, 1.0 );
		exp.add( 5.0, 1.0 );
		exp.add( 5.0, 3.0 );
		exp.add( 4.0, 4.0 );
		exp.add( 3.0, 3.0 );
		exp.add( 2.0, 4.0 );
		exp.add( 1.0, 3.0 );

		IPolygon result = Clip.intersection( p1, p2 );
		// result.print();
		assertEquals( exp, result );
	}

	/**
	 * Test the intersection of two complex, non-convex, non-self-intersecting
	 * polygons - cross four sides
	 */
	public void testIntersectionPolyFourSides( )
	{
		Polygon p1 = new Polygon();
		p1.add( 1.0, 0.5 );
		p1.add( 5.0, 0.5 );
		p1.add( 5.0, 6.0 );
		p1.add( 3.0, 4.0 );
		p1.add( 1.0, 6.0 );

		Polygon p2 = new Polygon();
		p2.add( 0.0, 2.0 );
		p2.add( 2.0, 0.0 );
		p2.add( 3.0, 1.0 );
		p2.add( 4.0, 0.0 );
		p2.add( 6.0, 2.0 );
		p2.add( 4.0, 4.0 );
		p2.add( 3.0, 3.0 );
		p2.add( 2.0, 4.0 );

		Polygon exp = new Polygon();
		exp.add( 1.0, 1.0 );
		exp.add( 1.5, 0.5 );
		exp.add( 2.5, 0.5 );
		exp.add( 3.0, 1.0 );
		exp.add( 3.5, 0.5 );
		exp.add( 4.5, 0.5 );
		exp.add( 5.0, 1.0 );
		exp.add( 5.0, 3.0 );
		exp.add( 4.0, 4.0 );
		exp.add( 3.0, 3.0 );
		exp.add( 2.0, 4.0 );
		exp.add( 1.0, 3.0 );

		IPolygon result = Clip.intersection( p1, p2 );
		assertEquals( exp, result );
	}

	/**
	 * Test the intersection of two complex, non-convex, non-self-intersecting
	 * polygons - V overlap
	 */
	public void testIntersectionPolyVOverlaps( )
	{
		Polygon p1 = new Polygon();
		p1.add( 1.0, 0.0 );
		p1.add( 5.0, 0.0 );
		p1.add( 5.0, 5.0 );
		p1.add( 3.0, 3.0 );
		p1.add( 1.0, 5.0 );

		Polygon p2 = new Polygon();
		p2.add( 0.0, 2.0 );
		p2.add( 2.0, 0.0 );
		p2.add( 3.0, 1.0 );
		p2.add( 4.0, 0.0 );
		p2.add( 6.0, 2.0 );
		p2.add( 4.0, 4.0 );
		p2.add( 3.0, 3.0 );
		p2.add( 2.0, 4.0 );

		Polygon exp = new Polygon();
		exp.add( 1.0, 1.0 );
		exp.add( 2.0, 0.0 );
		exp.add( 3.0, 1.0 );
		exp.add( 4.0, 0.0 );
		exp.add( 5.0, 1.0 );
		exp.add( 5.0, 3.0 );
		exp.add( 4.0, 4.0 );
		exp.add( 3.0, 3.0 );
		exp.add( 2.0, 4.0 );
		exp.add( 1.0, 3.0 );

		IPolygon result = Clip.intersection( p1, p2 );
		// result.print();
		assertEquals( exp, result );
	}

	// -------------
	// --- UNION ---
	// -------------
	/**
	 * Test the union of two polygons where the second is contained in the
	 * first.
	 */
	public void testUnionOneContainsTwo( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 0.0 );
		p1.add( 4.0, 0.0 );
		p1.add( 4.0, 4.0 );
		p1.add( 0.0, 4.0 );

		Polygon p2 = new Polygon();
		p2.add( 2.0, 1.0 );
		p2.add( 3.0, 1.0 );
		p2.add( 3.0, 3.0 );
		p2.add( 2.0, 3.0 );

		IPolygon result = Clip.union( p1, p2 );
		// result.print();
		assertEquals( p1, result );
	}

	/**
	 * Test the union of two polygons where the first is contained in the
	 * second.
	 */
	public void testUnionTwoContainsOne( )
	{
		Polygon p1 = new Polygon();
		p1.add( 2.0, 0.0 );
		p1.add( 3.0, 0.0 );
		p1.add( 3.0, 3.0 );
		p1.add( 2.0, 3.0 );

		Polygon p2 = new Polygon();
		p2.add( 0.0, 0.0 );
		p2.add( 4.0, 0.0 );
		p2.add( 4.0, 4.0 );
		p2.add( 0.0, 4.0 );

		IPolygon result = Clip.union( p1, p2 );
		// result.print();
		assertEquals( p2, result );
	}

	/**
	 * Test the union of two polygons that are equal.
	 */
	public void testUnionTwoEqual( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 0.0 );
		p1.add( 4.0, 0.0 );
		p1.add( 4.0, 4.0 );
		p1.add( 0.0, 4.0 );

		Polygon p2 = new Polygon();
		p2.add( 0.0, 0.0 );
		p2.add( 4.0, 0.0 );
		p2.add( 4.0, 4.0 );
		p2.add( 0.0, 4.0 );

		IPolygon result = Clip.union( p1, p2 );
		// result.print();
		assertEquals( p1, result );
	}

	/**
	 * Test the union of two rectangles that share one corner and two partial
	 * sides.
	 */
	public void testUnionRectCorner1( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 0.0 );
		p1.add( 4.0, 0.0 );
		p1.add( 4.0, 4.0 );
		p1.add( 0.0, 4.0 );

		Polygon p2 = new Polygon();
		p2.add( 0.0, 0.0 );
		p2.add( 1.0, 0.0 );
		p2.add( 1.0, 1.0 );
		p2.add( 0.0, 1.0 );

		Polygon exp = new Polygon();
		exp.add( 0.0, 0.0 );
		exp.add( 4.0, 0.0 );
		exp.add( 4.0, 4.0 );
		exp.add( 0.0, 4.0 );
		exp.add( 0.0, 1.0 ); // !!! KNOWN BUG - EXTRA POINT BUT SHAPE IS
								// CORRECT

		IPolygon result = Clip.union( p1, p2 );
		// result.print();
		assertEquals( exp, result );
	}

	/**
	 * Test the UNION of two rectangles that share one corner and two partial
	 * sides.
	 */
	public void testUnionRectCorner2( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 0.0 );
		p1.add( 4.0, 0.0 );
		p1.add( 4.0, 4.0 );
		p1.add( 0.0, 4.0 );

		Polygon p2 = new Polygon();
		p2.add( 3.0, 0.0 );
		p2.add( 4.0, 0.0 );
		p2.add( 4.0, 1.0 );
		p2.add( 3.0, 1.0 );

		Polygon exp = new Polygon();
		exp.add( 0.0, 0.0 );
		exp.add( 4.0, 0.0 );
		exp.add( 4.0, 1.0 ); // !!! KNOWN BUG - EXTRA POINT BUT SHAPE IS
								// CORRECT
		exp.add( 4.0, 4.0 );
		exp.add( 0.0, 4.0 );

		IPolygon result = Clip.union( p1, p2 );
		// result.print();
		assertEquals( exp, result );
	}

	/**
	 * Test the union of two rectangles that share one corner and two partial
	 * sides.
	 */
	public void testUnionRectCorner3( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 0.0 );
		p1.add( 4.0, 0.0 );
		p1.add( 4.0, 4.0 );
		p1.add( 0.0, 4.0 );

		Polygon p2 = new Polygon();
		p2.add( 3.0, 3.0 );
		p2.add( 4.0, 3.0 );
		p2.add( 4.0, 4.0 );
		p2.add( 3.0, 4.0 );

		Polygon exp = new Polygon();
		exp.add( 0.0, 0.0 );
		exp.add( 4.0, 0.0 );
		exp.add( 4.0, 3.0 ); // !!! KNOWN BUG - EXTRA POINT BUT SHAPE IS
								// CORRECT
		exp.add( 4.0, 4.0 );
		exp.add( 0.0, 4.0 );

		IPolygon result = Clip.union( p1, p2 );
		// result.print();
		assertEquals( exp, result );
	}

	/**
	 * Test the union of two rectangles that share one corner and two partial
	 * sides.
	 */
	public void testUnionRectCorner4( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 0.0 );
		p1.add( 4.0, 0.0 );
		p1.add( 4.0, 4.0 );
		p1.add( 0.0, 4.0 );

		Polygon p2 = new Polygon();
		p2.add( 0.0, 3.0 );
		p2.add( 1.0, 3.0 );
		p2.add( 1.0, 4.0 );
		p2.add( 0.0, 4.0 );

		Polygon exp = new Polygon();
		exp.add( 0.0, 0.0 );
		exp.add( 4.0, 0.0 );
		exp.add( 4.0, 4.0 );
		exp.add( 0.0, 4.0 );
		exp.add( 0.0, 3.0 ); // !!! KNOWN BUG - EXTRA POINT BUT SHAPE IS
								// CORRECT

		IPolygon result = Clip.union( p1, p2 );
		// result.print();
		assertEquals( exp, result );
	}

	/**
	 * Test the union of two rectangles that intersect on corner
	 */
	public void testUnionRectInterCorner1( )
	{
		Polygon p1 = new Polygon();
		p1.add( 2.0, 2.0 );
		p1.add( 4.0, 2.0 );
		p1.add( 4.0, 4.0 );
		p1.add( 2.0, 4.0 );

		Polygon p2 = new Polygon();
		p2.add( 1.0, 1.0 );
		p2.add( 3.0, 1.0 );
		p2.add( 3.0, 3.0 );
		p2.add( 1.0, 3.0 );

		Polygon exp = new Polygon();
		exp.add( 1.0, 1.0 );
		exp.add( 3.0, 1.0 );
		exp.add( 3.0, 2.0 );
		exp.add( 4.0, 2.0 );
		exp.add( 4.0, 4.0 );
		exp.add( 2.0, 4.0 );
		exp.add( 2.0, 3.0 );
		exp.add( 1.0, 3.0 );

		IPolygon result = Clip.union( p1, p2 );
		// result.print();
		assertEquals( exp, result );
	}

	/**
	 * Test the union of two rectangles that intersect on corner
	 */
	public void testUnionRectInterCorner2( )
	{
		Polygon p1 = new Polygon();
		p1.add( 2.0, 2.0 );
		p1.add( 4.0, 2.0 );
		p1.add( 4.0, 4.0 );
		p1.add( 2.0, 4.0 );

		Polygon p2 = new Polygon();
		p2.add( 3.0, 1.0 );
		p2.add( 5.0, 1.0 );
		p2.add( 5.0, 3.0 );
		p2.add( 3.0, 3.0 );

		Polygon exp = new Polygon();
		exp.add( 3.0, 1.0 );
		exp.add( 5.0, 1.0 );
		exp.add( 5.0, 3.0 );
		exp.add( 4.0, 3.0 );
		exp.add( 4.0, 4.0 );
		exp.add( 2.0, 4.0 );
		exp.add( 2.0, 2.0 );
		exp.add( 3.0, 2.0 );

		IPolygon result = Clip.union( p1, p2 );
		// result.print();
		assertEquals( exp, result );
	}

	/**
	 * Test the union of two rectangles that intersect on corner
	 */
	public void testUnionRectInterCorner3( )
	{
		Polygon p1 = new Polygon();
		p1.add( 2.0, 2.0 );
		p1.add( 4.0, 2.0 );
		p1.add( 4.0, 4.0 );
		p1.add( 2.0, 4.0 );

		Polygon p2 = new Polygon();
		p2.add( 3.0, 3.0 );
		p2.add( 5.0, 3.0 );
		p2.add( 5.0, 5.0 );
		p2.add( 3.0, 5.0 );

		Polygon exp = new Polygon();
		exp.add( 5.0, 3.0 );
		exp.add( 5.0, 5.0 );
		exp.add( 3.0, 5.0 );
		exp.add( 3.0, 4.0 );
		exp.add( 2.0, 4.0 );
		exp.add( 2.0, 2.0 );
		exp.add( 4.0, 2.0 );
		exp.add( 4.0, 3.0 );

		IPolygon result = Clip.union( p1, p2 );
		// result.print();
		assertEquals( exp, result );
	}

	/**
	 * Test the union of two rectangles that intersect on corner
	 */
	public void testUnionRectInterCorner4( )
	{
		Polygon p1 = new Polygon();
		p1.add( 2.0, 2.0 );
		p1.add( 4.0, 2.0 );
		p1.add( 4.0, 4.0 );
		p1.add( 2.0, 4.0 );

		Polygon p2 = new Polygon();
		p2.add( 1.0, 3.0 );
		p2.add( 3.0, 3.0 );
		p2.add( 3.0, 5.0 );
		p2.add( 1.0, 5.0 );

		Polygon exp = new Polygon();
		exp.add( 1.0, 3.0 );
		exp.add( 2.0, 3.0 );
		exp.add( 2.0, 2.0 );
		exp.add( 4.0, 2.0 );
		exp.add( 4.0, 4.0 );
		exp.add( 3.0, 4.0 );
		exp.add( 3.0, 5.0 );
		exp.add( 1.0, 5.0 );

		IPolygon result = Clip.union( p1, p2 );
		// result.print();
		assertEquals( exp, result );
	}

	/**
	 * Test the union of two rectangles where half of one is contained in the
	 * other and two sides of the inner cross one side of the outer.
	 */
	public void testUnionRectInterSide1( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 1.0 );
		p1.add( 4.0, 1.0 );
		p1.add( 4.0, 5.0 );
		p1.add( 0.0, 5.0 );

		Polygon p2 = new Polygon();
		p2.add( 1.0, 0.0 );
		p2.add( 3.0, 0.0 );
		p2.add( 3.0, 2.0 );
		p2.add( 1.0, 2.0 );

		Polygon exp = new Polygon();
		exp.add( 1.0, 0.0 );
		exp.add( 3.0, 0.0 );
		exp.add( 3.0, 1.0 );
		exp.add( 4.0, 1.0 );
		exp.add( 4.0, 5.0 );
		exp.add( 0.0, 5.0 );
		exp.add( 0.0, 1.0 );
		exp.add( 1.0, 1.0 );

		IPolygon result = Clip.union( p1, p2 );
		// result.print();
		assertEquals( exp, result );
	}

	/**
	 * Test the union of two rectangles where half of one is contained in the
	 * other and two sides of the inner cross one side of the outer.
	 */
	public void testUnionRectInterSide2( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 1.0 );
		p1.add( 4.0, 1.0 );
		p1.add( 4.0, 5.0 );
		p1.add( 0.0, 5.0 );

		Polygon p2 = new Polygon();
		p2.add( 3.0, 2.0 );
		p2.add( 5.0, 2.0 );
		p2.add( 5.0, 4.0 );
		p2.add( 3.0, 4.0 );

		Polygon exp = new Polygon();
		exp.add( 0.0, 1.0 );
		exp.add( 4.0, 1.0 );
		exp.add( 4.0, 2.0 );
		exp.add( 5.0, 2.0 );
		exp.add( 5.0, 4.0 );
		exp.add( 4.0, 4.0 );
		exp.add( 4.0, 5.0 );
		exp.add( 0.0, 5.0 );

		IPolygon result = Clip.union( p1, p2 );
		// result.print();
		assertEquals( exp, result );
	}

	/**
	 * Test the union of two rectangles where half of one is contained in the
	 * other and two sides of the inner cross one side of the outer.
	 */
	public void testUnionRectInterSide3( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 1.0 );
		p1.add( 4.0, 1.0 );
		p1.add( 4.0, 5.0 );
		p1.add( 0.0, 5.0 );

		Polygon p2 = new Polygon();
		p2.add( 1.0, 4.0 );
		p2.add( 3.0, 4.0 );
		p2.add( 3.0, 6.0 );
		p2.add( 1.0, 6.0 );

		Polygon exp = new Polygon();
		exp.add( 0.0, 1.0 );
		exp.add( 4.0, 1.0 );
		exp.add( 4.0, 5.0 );
		exp.add( 3.0, 5.0 );
		exp.add( 3.0, 6.0 );
		exp.add( 1.0, 6.0 );
		exp.add( 1.0, 5.0 );
		exp.add( 0.0, 5.0 );

		IPolygon result = Clip.union( p1, p2 );
		// result.print();
		assertEquals( exp, result );
	}

	/**
	 * Test the union of two rectangles where half of one is contained in the
	 * other and two sides of the inner cross one side of the outer.
	 */
	public void testUnionRectInterSide4( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 1.0 );
		p1.add( 4.0, 1.0 );
		p1.add( 4.0, 5.0 );
		p1.add( 0.0, 5.0 );

		Polygon p2 = new Polygon();
		p2.add( -1.0, 2.0 );
		p2.add( 1.0, 2.0 );
		p2.add( 1.0, 4.0 );
		p2.add( -1.0, 4.0 );

		Polygon exp = new Polygon();
		exp.add( 0.0, 1.0 );
		exp.add( 4.0, 1.0 );
		exp.add( 4.0, 5.0 );
		exp.add( 0.0, 5.0 );
		exp.add( 0.0, 4.0 );
		exp.add( -1.0, 4.0 );
		exp.add( -1.0, 2.0 );
		exp.add( 0.0, 2.0 );

		IPolygon result = Clip.union( p1, p2 );
		// result.print();
		assertEquals( exp, result );
	}

	/**
	 * Test the union of two complex, non-convex, non-self-intersecting polygons -
	 * two sides and one vertex - two triangles
	 */
	public void testUnionPolyTwoSidesOneVertex( )
	{
		Polygon p1 = new Polygon();
		p1.add( 1.0, 3.0 );
		p1.add( 5.0, 3.0 );
		p1.add( 5.0, 8.0 );
		p1.add( 3.0, 6.0 );
		p1.add( 1.0, 8.0 );

		Polygon p2 = new Polygon();
		p2.add( 0.0, 2.0 );
		p2.add( 2.0, 0.0 );
		p2.add( 3.0, 1.0 );
		p2.add( 4.0, 0.0 );
		p2.add( 6.0, 2.0 );
		p2.add( 4.0, 4.0 );
		p2.add( 3.0, 3.0 );
		p2.add( 2.0, 4.0 );

		Polygon exp = new Polygon();
		exp.add( 0.0, 2.0 );
		exp.add( 2.0, 0.0 );
		exp.add( 3.0, 1.0 );
		exp.add( 4.0, 0.0 );
		exp.add( 6.0, 2.0 );
		exp.add( 5.0, 3.0 );
		exp.add( 5.0, 8.0 );
		exp.add( 3.0, 6.0 );
		exp.add( 1.0, 8.0 );
		exp.add( 1.0, 3.0 );

		IPolygon result = Clip.union( p1, p2 );
		// result.print();
		assertEquals( exp, result );
	}

	/**
	 * Test the union of two complex, non-convex, non-self-intersecting polygons -
	 * two sides
	 */
	public void testUnionPolyTwoSides( )
	{
		Polygon p1 = new Polygon();
		p1.add( 1.0, 2.0 );
		p1.add( 5.0, 2.0 );
		p1.add( 5.0, 7.0 );
		p1.add( 3.0, 5.0 );
		p1.add( 1.0, 7.0 );

		Polygon p2 = new Polygon();
		p2.add( 0.0, 2.0 );
		p2.add( 2.0, 0.0 );
		p2.add( 3.0, 1.0 );
		p2.add( 4.0, 0.0 );
		p2.add( 6.0, 2.0 );
		p2.add( 4.0, 4.0 );
		p2.add( 3.0, 3.0 );
		p2.add( 2.0, 4.0 );

		Polygon exp = new Polygon();
		exp.add( 0.0, 2.0 );
		exp.add( 2.0, 0.0 );
		exp.add( 3.0, 1.0 );
		exp.add( 4.0, 0.0 );
		exp.add( 6.0, 2.0 );
		exp.add( 5.0, 3.0 );
		exp.add( 5.0, 7.0 );
		exp.add( 3.0, 5.0 );
		exp.add( 1.0, 7.0 );
		exp.add( 1.0, 3.0 );

		IPolygon result = Clip.union( p1, p2 );
		// result.print();
		assertEquals( exp, result );
	}

	/**
	 * Test the union of two complex, non-convex, non-self-intersecting polygons -
	 * two sides and one vertex - the lower one
	 */
	public void testUnionPolyTwoSidesAndLowerVertex( )
	{
		Polygon p1 = new Polygon();
		p1.add( 1.0, 1.0 );
		p1.add( 5.0, 1.0 );
		p1.add( 5.0, 6.0 );
		p1.add( 3.0, 4.0 );
		p1.add( 1.0, 6.0 );

		Polygon p2 = new Polygon();
		p2.add( 0.0, 2.0 );
		p2.add( 2.0, 0.0 );
		p2.add( 3.0, 1.0 );
		p2.add( 4.0, 0.0 );
		p2.add( 6.0, 2.0 );
		p2.add( 4.0, 4.0 );
		p2.add( 3.0, 3.0 );
		p2.add( 2.0, 4.0 );

		Polygon exp = new Polygon();
		exp.add( 0.0, 2.0 );
		exp.add( 2.0, 0.0 );
		exp.add( 3.0, 1.0 );
		exp.add( 4.0, 0.0 );
		exp.add( 6.0, 2.0 );
		exp.add( 5.0, 3.0 );
		exp.add( 5.0, 6.0 );
		exp.add( 3.0, 4.0 );
		exp.add( 1.0, 6.0 );
		exp.add( 1.0, 3.0 );

		IPolygon result = Clip.union( p1, p2 );
		// result.print();
		assertEquals( exp, result );
	}

	/**
	 * Test the union of two complex, non-convex, non-self-intersecting polygons -
	 * cross four sides
	 */
	public void testUnionPolyFourSides( )
	{
		Polygon p1 = new Polygon();
		p1.add( 1.0, 0.5 );
		p1.add( 5.0, 0.5 );
		p1.add( 5.0, 6.0 );
		p1.add( 3.0, 4.0 );
		p1.add( 1.0, 6.0 );

		Polygon p2 = new Polygon();
		p2.add( 0.0, 2.0 );
		p2.add( 2.0, 0.0 );
		p2.add( 3.0, 1.0 );
		p2.add( 4.0, 0.0 );
		p2.add( 6.0, 2.0 );
		p2.add( 4.0, 4.0 );
		p2.add( 3.0, 3.0 );
		p2.add( 2.0, 4.0 );

		Polygon exp = new Polygon();
		exp.add( 1.0, 1.0 );
		exp.add( 1.0, 0.5 );
		exp.add( 1.5, 0.5 );
		exp.add( 2.0, 0.0 );
		exp.add( 2.5, 0.5 );
		exp.add( 3.5, 0.5 );
		exp.add( 4.0, 0.0 );
		exp.add( 4.5, 0.5 );
		exp.add( 5.0, 0.5 );
		exp.add( 5.0, 1.0 );
		exp.add( 6.0, 2.0 );
		exp.add( 5.0, 3.0 );
		exp.add( 5.0, 6.0 );
		exp.add( 3.0, 4.0 );
		exp.add( 1.0, 6.0 );
		exp.add( 1.0, 3.0 );
		exp.add( 0.0, 2.0 );

		IPolygon result = Clip.union( p1, p2 );
		// result.print();
		assertEquals( exp, result );
	}

	/**
	 * Test the union of two complex, non-convex, non-self-intersecting polygons -
	 * V overlap
	 */
	public void testUnionPolyVOverlaps( )
	{
		Polygon p1 = new Polygon();
		p1.add( 1.0, 0.0 );
		p1.add( 5.0, 0.0 );
		p1.add( 5.0, 5.0 );
		p1.add( 3.0, 3.0 );
		p1.add( 1.0, 5.0 );

		Polygon p2 = new Polygon();
		p2.add( 0.0, 2.0 );
		p2.add( 2.0, 0.0 );
		p2.add( 3.0, 1.0 );
		p2.add( 4.0, 0.0 );
		p2.add( 6.0, 2.0 );
		p2.add( 4.0, 4.0 );
		p2.add( 3.0, 3.0 );
		p2.add( 2.0, 4.0 );

		Polygon exp = new Polygon();
		exp.add( 1.0, 0.0 );
		exp.add( 5.0, 0.0 );
		exp.add( 5.0, 1.0 );
		exp.add( 6.0, 2.0 );
		exp.add( 5.0, 3.0 );
		exp.add( 5.0, 5.0 );
		exp.add( 4.0, 4.0 ); // KNOWN BUG - EXTRA POINT BUT SHAPE IS OK
		exp.add( 3.0, 3.0 );
		exp.add( 2.0, 4.0 ); // KNOWN BUG - EXTRA POINT BUT SHAPE IS OK
		exp.add( 1.0, 5.0 );
		exp.add( 1.0, 3.0 );
		exp.add( 0.0, 2.0 );
		exp.add( 1.0, 1.0 );

		IPolygon result = Clip.union( p1, p2 );
		// result.print();
		assertEquals( exp, result );
	}

	// -----------
	// --- XOR ---
	// -----------
	/**
	 * Test the xor of two polygons that are equal.
	 */
	public void testXorTwoEqual( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 0.0 );
		p1.add( 4.0, 0.0 );
		p1.add( 4.0, 4.0 );
		p1.add( 0.0, 4.0 );

		Polygon p2 = new Polygon();
		p2.add( 0.0, 0.0 );
		p2.add( 4.0, 0.0 );
		p2.add( 4.0, 4.0 );
		p2.add( 0.0, 4.0 );

		IPolygon result = Clip.xor( p1, p2 );
		// result.print();
		assertTrue( result.isEmpty( ) );
	}

	/**
	 * Test the xor of two rectangles that share one corner and two partial
	 * sides.
	 */
	public void testXorRectCorner1( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 0.0 );
		p1.add( 4.0, 0.0 );
		p1.add( 4.0, 4.0 );
		p1.add( 0.0, 4.0 );

		Polygon p2 = new Polygon();
		p2.add( 0.0, 0.0 );
		p2.add( 1.0, 0.0 );
		p2.add( 1.0, 1.0 );
		p2.add( 0.0, 1.0 );

		Polygon exp = new Polygon();
		exp.add( 1.0, 0.0 );
		exp.add( 4.0, 0.0 );
		exp.add( 4.0, 4.0 );
		exp.add( 0.0, 4.0 );
		exp.add( 0.0, 1.0 );
		exp.add( 1.0, 1.0 );

		IPolygon result = Clip.xor( p1, p2 );
		// result.print();
		assertEquals( exp, result );
	}

	/**
	 * Test the xor of two rectangles that share one corner and two partial
	 * sides.
	 */
	public void testXorRectCorner2( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 0.0 );
		p1.add( 4.0, 0.0 );
		p1.add( 4.0, 4.0 );
		p1.add( 0.0, 4.0 );

		Polygon p2 = new Polygon();
		p2.add( 3.0, 0.0 );
		p2.add( 4.0, 0.0 );
		p2.add( 4.0, 1.0 );
		p2.add( 3.0, 1.0 );

		Polygon exp = new Polygon();
		exp.add( 0.0, 0.0 );
		exp.add( 3.0, 0.0 );
		exp.add( 3.0, 1.0 );
		exp.add( 4.0, 1.0 );
		exp.add( 4.0, 4.0 );
		exp.add( 0.0, 4.0 );

		IPolygon result = Clip.xor( p1, p2 );
		// result.print();
		assertEquals( exp, result );
	}

	/**
	 * Test the xor of two rectangles that share one corner and two partial
	 * sides.
	 */
	public void testXorRectCorner3( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 0.0 );
		p1.add( 4.0, 0.0 );
		p1.add( 4.0, 4.0 );
		p1.add( 0.0, 4.0 );

		Polygon p2 = new Polygon();
		p2.add( 3.0, 3.0 );
		p2.add( 4.0, 3.0 );
		p2.add( 4.0, 4.0 );
		p2.add( 3.0, 4.0 );

		Polygon exp = new Polygon();
		exp.add( 0.0, 0.0 );
		exp.add( 4.0, 0.0 );
		exp.add( 4.0, 3.0 );
		exp.add( 3.0, 3.0 );
		exp.add( 3.0, 4.0 );
		exp.add( 0.0, 4.0 );

		IPolygon result = Clip.xor( p1, p2 );
		// result.print();
		assertEquals( exp, result );
	}

	/**
	 * Test the xor of two rectangles that share one corner and two partial
	 * sides.
	 */
	public void testXorRectCorner4( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 0.0 );
		p1.add( 4.0, 0.0 );
		p1.add( 4.0, 4.0 );
		p1.add( 0.0, 4.0 );

		Polygon p2 = new Polygon();
		p2.add( 0.0, 3.0 );
		p2.add( 1.0, 3.0 );
		p2.add( 1.0, 4.0 );
		p2.add( 0.0, 4.0 );

		Polygon exp = new Polygon();
		exp.add( 0.0, 0.0 );
		exp.add( 4.0, 0.0 );
		exp.add( 4.0, 4.0 );
		exp.add( 1.0, 4.0 );
		exp.add( 1.0, 3.0 );
		exp.add( 0.0, 3.0 );

		IPolygon result = Clip.xor( p1, p2 );
		// result.print();
		assertEquals( exp, result );
	}

	/**
	 * Test the xor of two rectangles where half of one is contained in the
	 * other and two sides of the inner cross one side of the outer.
	 */
	public void testXorRectInterSide1( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 1.0 );
		p1.add( 4.0, 1.0 );
		p1.add( 4.0, 5.0 );
		p1.add( 0.0, 5.0 );

		Polygon p2 = new Polygon();
		p2.add( 1.0, 0.0 );
		p2.add( 3.0, 0.0 );
		p2.add( 3.0, 2.0 );
		p2.add( 1.0, 2.0 );

		Polygon exp = new Polygon();
		exp.add( 0.0, 1.0 );
		exp.add( 1.0, 1.0 );
		exp.add( 1.0, 0.0 );
		exp.add( 3.0, 0.0 );
		exp.add( 3.0, 1.0 );
		exp.add( 1.0, 1.0 );
		exp.add( 1.0, 2.0 );
		exp.add( 3.0, 2.0 );
		exp.add( 3.0, 1.0 );
		exp.add( 4.0, 1.0 );
		exp.add( 4.0, 5.0 );
		exp.add( 0.0, 5.0 );

		IPolygon result = Clip.xor( p1, p2 );
		// result.print();
		assertEquals( exp, result );
	}

	/**
	 * Test the xor of two rectangles where half of one is contained in the
	 * other and two sides of the inner cross one side of the outer.
	 */
	public void testXornRectInterSide2( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 1.0 );
		p1.add( 4.0, 1.0 );
		p1.add( 4.0, 5.0 );
		p1.add( 0.0, 5.0 );

		Polygon p2 = new Polygon();
		p2.add( 3.0, 2.0 );
		p2.add( 5.0, 2.0 );
		p2.add( 5.0, 4.0 );
		p2.add( 3.0, 4.0 );

		Polygon exp = new Polygon();
		exp.add( 4.0, 5.0 );
		exp.add( 0.0, 5.0 );
		exp.add( 0.0, 1.0 );
		exp.add( 4.0, 1.0 );
		exp.add( 4.0, 2.0 );
		exp.add( 3.0, 2.0 );
		exp.add( 3.0, 4.0 );
		exp.add( 4.0, 4.0 );
		exp.add( 4.0, 2.0 );
		exp.add( 5.0, 2.0 );
		exp.add( 5.0, 4.0 );
		exp.add( 4.0, 4.0 );

		IPolygon result = Clip.xor( p1, p2 );
		// result.print();
		assertEquals( exp, result );
	}

	/**
	 * Test the xor of two rectangles where half of one is contained in the
	 * other and two sides of the inner cross one side of the outer.
	 */
	public void testXorRectInterSide3( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 1.0 );
		p1.add( 4.0, 1.0 );
		p1.add( 4.0, 5.0 );
		p1.add( 0.0, 5.0 );

		Polygon p2 = new Polygon();
		p2.add( 1.0, 4.0 );
		p2.add( 3.0, 4.0 );
		p2.add( 3.0, 6.0 );
		p2.add( 1.0, 6.0 );

		Polygon exp = new Polygon();
		exp.add( 4.0, 5.0 );
		exp.add( 3.0, 5.0 );
		exp.add( 3.0, 6.0 );
		exp.add( 1.0, 6.0 );
		exp.add( 1.0, 5.0 );
		exp.add( 3.0, 5.0 );
		exp.add( 3.0, 4.0 );
		exp.add( 1.0, 4.0 );
		exp.add( 1.0, 5.0 );
		exp.add( 0.0, 5.0 );
		exp.add( 0.0, 1.0 );
		exp.add( 4.0, 1.0 );

		IPolygon result = Clip.xor( p1, p2 );
		// result.print();
		assertEquals( exp, result );
	}

	/**
	 * Test the xor of two rectangles where half of one is contained in the
	 * other and two sides of the inner cross one side of the outer.
	 */
	public void testXorRectInterSide4( )
	{
		Polygon p1 = new Polygon();
		p1.add( 0.0, 1.0 );
		p1.add( 4.0, 1.0 );
		p1.add( 4.0, 5.0 );
		p1.add( 0.0, 5.0 );

		Polygon p2 = new Polygon();
		p2.add( -1.0, 2.0 );
		p2.add( 1.0, 2.0 );
		p2.add( 1.0, 4.0 );
		p2.add( -1.0, 4.0 );

		Polygon exp = new Polygon();
		exp.add( 4.0, 5.0 );
		exp.add( 0.0, 5.0 );
		exp.add( 0.0, 4.0 );
		exp.add( 1.0, 4.0 );
		exp.add( 1.0, 2.0 );
		exp.add( 0.0, 2.0 );
		exp.add( 0.0, 4.0 );
		exp.add( -1.0, 4.0 );
		exp.add( -1.0, 2.0 );
		exp.add( 0.0, 2.0 );
		exp.add( 0.0, 1.0 );
		exp.add( 4.0, 1.0 );

		IPolygon result = Clip.xor( p1, p2 );
		// result.print();
		assertEquals( exp, result );
	}
}
