/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.chart.tests.engine.util;

import junit.framework.TestCase;
import org.eclipse.birt.chart.util.QRDecomposition;
import org.eclipse.birt.chart.util.Matrix;

public class QRDecompositionTest extends TestCase
{

	Matrix matrix1, matrix2, matrix3;
	QRDecomposition qrd1, qrd2, qrd3;

	/**
	 * Construct and initialize any objects that will be used in multiple tests.
	 */
	protected void setUp( ) throws Exception
	{
		super.setUp( );
		matrix1 = new Matrix( 2, 2 );
		qrd1 = new QRDecomposition( matrix1 );

		matrix2 = new Matrix( 2, 2, 2.0 );
		qrd2 = new QRDecomposition( matrix2 );

		matrix3 = new Matrix( 4, 4, 1.0 );
		qrd3 = new QRDecomposition( matrix3 );
	}

	/**
	 * Collect and empty any objects that are used in multiple tests.
	 */
	protected void tearDown( ) throws Exception
	{
		super.tearDown( );
		matrix1 = null;
		matrix2 = null;
		matrix3 = null;
		qrd1 = null;
		qrd2 = null;
		qrd3 = null;
	}

	/**
	 * Test the array for internal storage of diagonal of R.
	 *
	 */
	public void testHypot( )
	{
		assertTrue( QRDecomposition.hypot( -4, 3 ) == 5 );
		assertTrue( QRDecomposition.hypot( 3, 4 ) == 5 );
		assertTrue( QRDecomposition.hypot( 0, 0 ) == 0 );
	}

	public void testIsFullRank( )
	{
		assertFalse( qrd1.isFullRank( ) );
		assertTrue( qrd2.isFullRank( ) );
		assertFalse( qrd3.isFullRank( ) );
	}

	/**
	 * Test the Householder vectors.
	 *
	 */
	public void testGetH( )
	{
		Matrix X = qrd2.getH( );
		double[][] A = new double[2][2];
		A[0][0] = 1.0 + Math.sqrt( 0.5 );
		A[0][1] = 0.0;
		A[1][0] = A[0][0] - 1.0;
		A[1][1] = 2.0;

		for ( int i = 0; i < 2; i++ )
		{
			for ( int j = 0; j < 2; j++ )
			{
				assertTrue( X.get( i, j ) == A[i][j] );
			}
		}
	}

	/**
	 * Test the upper triangular factor.
	 *
	 */
	public void testGetR( )
	{
		Matrix X = qrd3.getR( );

		for ( int j = 0; j < 4; j++ )
		{
			assertTrue( X.get( 0, j ) == -2.0 );
		}

		for ( int i = 1; i < 4; i++ )
		{
			for ( int j = 0; j < 4; j++ )
			{
				if ( i == j )
				{
					assertTrue( X.get( i, j ) == -0.0 );
				}
				else
				{
					assertTrue( X.get( i, j ) == 0.0 );
				}
			}
		}
	}

	/**
	 * Test the (economy-sized) orthogonal factor.
	 *
	 */
	public void testGetQ( )
	{
		Matrix X = qrd2.getQ( );
		double[][] A = new double[2][2];
		A[0][0] = -( 1.0 + Math.sqrt( 0.5 ) - 1.0 );
		A[0][1] = -A[0][0];
		A[1][0] = A[0][0];
		A[1][1] = -Math.sqrt( 0.5 );

		for ( int i = 0; i < 2; i++ )
		{
			for ( int j = 0; j < 2; j++ )
			{
				assertTrue( X.get( i, j ) == A[i][j] );
			}
		}
	}

	/**
	 * Test the solution of A*X = B.
	 *
	 */
	public void testSolve( )
	{
		Matrix X = qrd2.solve( matrix2 );
		double[][] A = new double[2][2];
		A[0][0] = -0.0;
		A[0][1] = -0.0;
		A[1][0] = 1.0;
		A[1][1] = 1.0;

		for ( int i = 0; i < 2; i++ )
		{
			for ( int j = 0; j < 2; j++ )
			{
				assertTrue( X.get( i, j ) == A[i][j] );
			}
		}
	}
}
